package de.fraunhofer.iem.dataprovider

import de.fraunhofer.iem.dataprovider.taskManager.Worker
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.tasks.Task
import io.mockk.mockk
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.test.runTest
import org.junit.jupiter.api.Test
import java.util.concurrent.atomic.AtomicInteger

class WorkerTest {


    private class ExceptionTask(override val responseChannel: suspend (event: Event) -> Unit) : Task() {
        override suspend fun execute() {
            throw Exception("Execution crashed!")
        }
    }

    private class WaitingTask(override val responseChannel: suspend (event: Event) -> Unit) : Task() {
        override suspend fun execute() {
            val testEvent = mockk<Event>()
            responseChannel(testEvent)
        }
    }

    @OptIn(ExperimentalCoroutinesApi::class)
    @Test
    fun workerExceptionTest() = runTest {
        val worker = Worker("Worker", this)

        val counter = AtomicInteger()

        suspend fun responseChannelDummy(event: Event): Unit {
            val curr = counter.incrementAndGet()
            println("counter $counter")
            if (curr == 2) {
                worker.close()
            }
        }
        worker.addTask(WaitingTask(::responseChannelDummy))
        worker.addTask(ExceptionTask(::responseChannelDummy))
        worker.addTask(WaitingTask(::responseChannelDummy))
    }
}
